function [gainh,gainv] = im_edges(xui,thresh)
% function [gainh,gainv] = im_edges(xui,thresh)
%
% Calculate gain matrices corresponding to the edges of image xui.
% gainh contains gains which depend on the horizontal gradients of xui
% and tends to zero where the gradients are << thresh and to one where
% the gradients are >> thresh. 
% Similarly for gainv and vertical gradients.
% If thresh is a vector, then separate planes of gainh and gainv are
% created for each threshold element.
%
% Nick Kingsbury, Cambridge University, 2006.

% Calculate luminance image.
L1 = 0.3*double(xui(:,:,1)) + 0.6*double(xui(:,:,2)) + 0.1*double(xui(:,:,3)); 

% Calculate edges.
edv = diff(colfilter(L1,gaussian(0.8)));
edh = diff(colfilter(L1.',gaussian(0.8))).';

% Calculate vertical and horizontal gain matrices for each threshold.
K = length(thresh);
gainv = zeros([size(edv) K]);
gainh = zeros([size(edh) K]);
for k = 1:K, % Loop for each threshold k (K should equal 2).

    % Calculate the gain matrices for threshold(k).
    c = 1 / thresh(k).^2;
    gainv(:,:,k) = ones(size(edv)) ./ (1 + c*edv.*edv);
    gainh(:,:,k) = ones(size(edh)) ./ (1 + c*edh.*edh);

    % Plot the gain matrices as greyscale images.
    figure(k+2);
    set(gcf,'position',[100  30+(2-k)*100   910   470]);
    set(gcf,'numbertitle','off','name','Edge gains');
    subplot('Position',[0.04 0.05 0.44 0.9]);
    image(gainv(:,:,k)*255 + 1); colormap(gray(256)); 
    axis image; axis ij;
    if k==1, title('Vertical Edge Gain - Denoising (white areas)',...
            'FontSize',12);
    else title('Vertical Edge Gain - Sharpening (black areas)',...
            'FontSize',12);
    end
    subplot('Position',[0.54 0.05 0.44 0.9]);
    image(gainh(:,:,k)*255 + 1); colormap(gray(256)); 
    axis image; axis ij;
    if k==1, title('Horizontal Edge Gain - Denoising (white areas)',...
            'FontSize',12);
    else title('Horizontal Edge Gain - Sharpening (black areas)',...
            'FontSize',12);
    end
    drawnow
end

figure(2);
return
