function [yui,xshift] = im_morph(xui,xshift,cp,waitstr)

% function [yui,xshift] = im_morph(xui,xshift,cp,waitstr)
%
% Morph xui, so that pixels near control points cp(:,1:2) are moved to cp(:,3:4).
% The region of influence is a set of Gaussian blobs whose std dev = cp(:,5).
% xshift is the shift matrix that accumulates the effects of each blob.
% Set xshift = 0 on the first call to im_morph.
% waitstr is the displayed title of the wait bar.
%
% Nick Kingsbury, Cambridge University, 2006.

if nargin < 4, waitstr = 'Morphing image'; end
wh = waitbar(0,waitstr);

sx = size(xui);
s = [1:sx(1)].';
t = [1:sx(2)];
o1 = ones(sx(1),1);
o2 = ones(1,sx(2));

% If extra morph points are given, add their effects to xshift.
if (nargin > 2) & (~isempty(cp)),
    sc = size(cp); % cp must be of width 5.
    for nc = 1:sc(1),  % Loop for each pair of points to be morphed.
      % Ensure that ri and ci have all elements still in monotonically increasing
      % order in each direction, by increasing rad(nc) if necessary.
      rad = max(cp(nc,5),0.8*sqrt(sum((cp(nc,3:4)-cp(nc,1:2)).^2)));
      % Generate the Gaussian blob, centred on c1(nc,1:2) and of radius rad(nc).
      gblob = exp(-(0.5/rad.^2)*((s-cp(nc,4)).^2*o2 + o1*(t-cp(nc,3)).^2));
      % Scale the blob by (c0(nc,:)-c1(nc,:)) and add it to xshift.
      xshift = xshift + gblob*(cp(nc,1) - cp(nc,3) + j*(cp(nc,2)-cp(nc,4)));
    end
end

% ri and ci are the matrices of the row and column indices of the
% locations in xui from which each interpolated point in yui is to come.
ri = max(min(s*o2 + imag(xshift),sx(1)),1);
ci = max(min(o1*t + real(xshift),sx(2)),1);

% Perform the morphing using interp2().
yui = uint8(zeros(sx(1),sx(2),3));
for k = 1:3,
    yui(:,:,k) = uint8(interp2(double(xui(:,:,k)),ci,ri,'linear') + 0.5);
    waitbar(k/3);
end
close(wh) % Close wait bar
return
